var aoiDropVer = '1.0.1';

// AoiDropステータス取得
function getAppStatus() {
  return 'AoiDropは正常に稼働しています(/＞v＜)/' + ' ' + 'バージョン：' + aoiDropVer;
}

// チェック処理
function check(trackNum, hasText) {
  var count = 0;
  var trackCount = trackNum + 1;
  if (!app.isDocumentOpen()) return 'Error UnOpened Project';
  if (app.project.activeSequence == 0) return 'Error No sequence';
  if (app.project.activeSequence.audioTracks.length < trackCount) return 'Error Few AudioTrack';
  if (hasText) {
    if (app.project.activeSequence.videoTracks.length < trackCount) return 'Error Few VideoTrack';
  }
  return '';
}

// 「voice」ビン取得
function findVoiceBin() {
  var childrenItem = app.project.rootItem.children;
  for (var i = 0; i <= childrenItem.length; i++) {
    var childItem = childrenItem[i];
    if(!childItem) continue;
    if (childItem.type === 2 && childItem.name.replace(' ', '') === 'voice') {
      return childItem;
    }
  }
  // 「voice」ビンがない場合、作成
  return app.project.rootItem.createBin('voice');
}

// モーショングラフィックステンプレートメディアビンを取得
function findMogrtBin() {
  var childrenItem = app.project.rootItem.children;
  for (var i = 0; i <= childrenItem.length; i++) {
    var childItem = childrenItem[i];
    if(!childItem) continue;
    if (childItem.type === 2 && childItem.name === 'モーショングラフィックステンプレートメディア') {
      return childItem;
    }
  }
  return null;
}

// 最後のクリップの末尾の時間を取得
function getEndClipTime(activeSeq, targetTime, trackNum, hasText) {
  var endClipTime = targetTime;
  var audioClips = activeSeq.audioTracks[trackNum].clips;
  for (var i = 0; i <= audioClips.length; i++) {
    var audioClip = audioClips[i];
    if(!audioClip) continue;
    if(endClipTime.seconds < audioClip.end.seconds) endClipTime = audioClip.end;
  }
  if (hasText) {
    var videoClips = activeSeq.videoTracks[trackNum].clips;
    for (var i = 0; i <= videoClips.length; i++) {
      var videoClip = videoClips[i];
      if(!videoClip) continue;
      if(endClipTime.seconds < videoClip.end.seconds) endClipTime = videoClip.end;
    }
  }
  return endClipTime;
}

// wav読込
function importWav(voiceBin, wavPath, wavName) {
  // 音声インポート
  if (!app.project.importFiles(wavPath, true, voiceBin, false)) return null;
  // インポートした音声を探す
  var voiceItems = voiceBin.children;
  for (var i = 0; i <= voiceItems.length; i++) {
    var voiceItem = voiceItems[i];
    if(!voiceItem) continue;
    if (voiceItem.type === 1 && voiceItem.name === wavName) {
      return voiceItem;
    }
  }
  return null;
}

// wav配置
function dropWav(activeSeq, wavName, voiceItem, targetTime, trackNum) {
  // 音声配置
  activeSeq.audioTracks[trackNum].insertClip(voiceItem, targetTime, 0, trackNum);
  // 配置した音声を探す
  var voiceClips = activeSeq.audioTracks[trackNum].clips;
  for (var i = 0; i <= voiceClips.length; i++) {
    var voiceClip = voiceClips[i];
    if(!voiceClip) continue;
    if (voiceClip.name === wavName) {
      return voiceClip;
    }
  }
  return null;
}

// text配置
function dropText(activeSeq, mogrtPath, text, targetTime, trackNum, wavEndPosition) {
  // テンプレテキスト読み込み
  var textClip = activeSeq.importMGT(mogrtPath, targetTime, trackNum, 0);
  if (!textClip) return null;
  var textComponent = textClip.getMGTComponent();
  if (!textComponent) return null;
  var textParams = textComponent.properties;
  var srcTextParam = textParams.getParamForDisplayName(textParams[0].displayName);
  if (!srcTextParam) return null;
  // テキスト書き換え
  srcTextParam.setValue(text);
  // 表示文書き換え(最大20文字)
  if (text.length > 20) textClip.name = text.slice(0, 19);
  else textClip.name = text;
  // クリップの長さ調整
  textClip.end = wavEndPosition;
  // textClip.outPoint = wavEndPosition * 2;
  return textClip;
}

// クリップをリンク
function linkClip(activeSeq, voiceClip, textClip) {
  // 全クリップの選択を解除
  var audioTracks = activeSeq.audioTracks;
  for (var i = 0; i <= audioTracks.length; i++) {
    var audioClips = audioTracks[i];
    if(!audioClips) continue;
    for (var j = 0; j <= audioClips.clips.length; j++) {
      var audioClip = audioClips.clips[j];
      if(!audioClip) continue;
      if (audioClip.isSelected()) audioClip.setSelected(0, 1);
    }
  }
  var videoTracks = activeSeq.videoTracks;
  for (var i = 0; i <= videoTracks.length; i++) {
    var videoClips = videoTracks[i];
    if(!videoClips) continue;
    for (var j = 0; j <= videoClips.clips.length; j++) {
      var videoClip = videoClips.clips[j];
      if(!videoClip) continue;
      if (videoClip.isSelected()) videoClip.setSelected(0, 1);
    }
  }
  // 音声、字幕を選択状態にし、リンク
  voiceClip.setSelected(1, 1);
  textClip.setSelected(1, 1);
  activeSeq.linkSelection();
}

// 元のビンに戻る
function returnBin(insertionBin) {
  // 最初に表示していたビンにダミービンを作成する
  var dummyBin = insertionBin.createBin('dummydummy');
  // ダミービンを消して最初に表示していたビンに戻る
  dummyBin.deleteBin();
}

// メイン処理
function dropExecute(wavPath, wavName, mogrtPath, txtPath, trackNum, clipLink) {
  var hasText = true;
  if (txtPath === '') hasText = false;
  var checkResult = check(trackNum, hasText);
  if (checkResult !== '') return checkResult;
  var activeSeq = app.project.activeSequence;
  var voiceBin = findVoiceBin();
  var targetTime = activeSeq.getPlayerPosition();

  // 現在表示しているビン
  var insertionBin = app.project.getInsertionBin();

  // 末尾のクリップの時間を取得
  var endClipTime = getEndClipTime(activeSeq, targetTime, trackNum, hasText);

  // 音声読み込み
  var voiceItem = importWav(voiceBin, wavPath, wavName);
  if(voiceItem === null) return 'Error Drop Voice';
  // 音声投げ込み
  var voiceClip = dropWav(activeSeq, wavName, voiceItem, endClipTime, trackNum);
  if(voiceClip === null) return 'Error Drop Voice';
  var wavEndPosition = voiceClip.end;

  // カーソル位置を動かす
  activeSeq.setPlayerPosition(wavEndPosition.ticks);

  // 音声のみの場合、ここで終了
  if (!hasText) {
    returnBin(insertionBin);
    return 'Succeeded';
  }

  // テキスト読み込み
  var file = new File(txtPath);
  file.encoding = "UTF8";
  file.open("r", "TEXT", "????");
  // テキスト投げ込み
  var textClip = dropText(activeSeq, mogrtPath, file.read(), endClipTime, trackNum, wavEndPosition);
  if (textClip === null) return 'Error Drop Text';

  // クリップをリンク
  if (clipLink === 1) linkClip(activeSeq, voiceClip, textClip);

  // 元のビンに戻る
  returnBin(insertionBin);
  
  return 'Succeeded';
}